@@ -21,36 +21,12 @@
 
 #include <cstdlib>
 #include <ext/pool_allocator.h>
-#include <testsuite_hooks.h>
-#include <testsuite_allocator.h>
-
-using __gnu_cxx::__pool_alloc;
-
-void* 
-operator new(std::size_t n) throw(std::bad_alloc)
-{
-  new_called = true;
-  return std::malloc(n);
-}
-
-void
-operator delete(void *v) throw()
-{
-  delete_called = true;
-  return std::free(v);
-}
-
-void test01() 
-{ 
-  // Uses new, but delete only sometimes.
-  bool test __attribute__((unused)) = true;
-  typedef __pool_alloc<unsigned int> allocator_type;
-  VERIFY( bool(__gnu_test::check_delete<allocator_type, false>()) ); 
-}
+#include <replacement_memory_operators.h>
 
 int main()
 {
-  test01();
+  // Uses new, but delete only sometimes.
+  typedef __gnu_cxx::__pool_alloc<unsigned int> allocator_type;
+  __gnu_test::check_delete<allocator_type, false>(); 
   return 0;
 }
-
